#!/usr/bin/env python

import ete3
import argparse

#This function will obtain a dictionary with the rooting order based on the species tree.
def get_spe2age(t,seed):
	spe2age = {}
	node = t.get_leaves_by_name(seed)[0]
	age = 1
	while not node.is_root():
		for leaf in node.get_leaves():
			if leaf.name not in spe2age:
				spe2age[leaf.name] = age
		age += 1
		node = node.up
	for leaf in t.get_leaves():
		if leaf.name not in spe2age:
			spe2age[leaf.name] = age
	return spe2age

#This function will load the species name into ETE
def get_species_name(node):
	return node.split("_")[1]

#This function will load the whole name into ETE
def get_whole_name(node):
	return node


parser = argparse.ArgumentParser(description="Obtains pairs of BRH")
parser.add_argument("-t",dest="treeFile",action="store",required=True,help="List of gene trees")
parser.add_argument("-s",dest="spTreeFile",action="store",required=True,help="Species tree file")
parser.add_argument("-n",dest="tag",action="store",required=True,help="Species code of your species of interest")
parser.add_argument("-o",dest="outfileName",action="store",required=True,help="File where the results will be printed")
args = parser.parse_args()

#Load species tree
spTree = ete3.PhyloTree(args.spTreeFile,sp_naming_function=get_whole_name)
#Obtain the rooting preference dictionary
spe2age = get_spe2age(spTree,args.tag)

#Open the results file
outfile = open(args.outfileName,"w")
#For each line in the file (each line is formated as seed_code<tab>tree)
for line in open(args.treeFile):
	line = line.strip()
	#We split the line
	dades = line.split("\t")
	seedCode = dades[0]
	#Load the gene tree
	t = ete3.PhyloTree(dades[1],sp_naming_function=get_species_name)
	#Select the outgroup
	outgroup = t.get_farthest_oldest_leaf(spe2age)
	#Set the outgroup
	t.set_outgroup(outgroup)
	#Choose the seed node
	seedNode = t.get_leaves_by_name(seedCode)[0]
	#Detect the evolutionary events that pertain to the seed protein
	events = seedNode.get_my_evol_events()
	#Iter the events
	orthologs = {}
	for ev in events:
		#Select the speciation events
		if ev.etype == "S":
			#Choose the two children and select the one where the seed is not present
			children = ev.node.get_children()
			if seedCode not in children[0]:
				child = children[0]
			else:
				child = children[1]
			#For the seed iter the different leaves and store them into memory depending on their species
			for node in child.iter_leaves():
				spe = node.species
				if spe not in orthologs:
					orthologs[spe] = set([])
				orthologs[spe].add(node.name)
	#Once the whole tree has been processed choose those species that have only one ortholog and print that information into a file.
	for spe in orthologs:
		if len(orthologs[spe]) == 1:
			print >>outfile,seedCode+"\t"+spe+"\t"+list(orthologs[spe])[0]
outfile.close()			
	
	
